/*
Copyright (C) BABEC. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/
package org.chainweaver.did.sdk.model;


import com.google.gson.*;

import java.lang.reflect.Type;

/**
 * 登录token
 */
public class AccessToken {

    private String accessToken;

    private Long expiresIn;

    private long userId;

    private String did;

    private String userName;

    private String enterpriseCName;

    private Integer memberType;

    private Integer isOperatingCenter;

    public String getAccessToken() {
        return accessToken;
    }

    public void setAccessToken(String accessToken) {
        this.accessToken = accessToken;
    }

    public Long getExpiresIn() {
        return expiresIn;
    }

    public void setExpiresIn(Long expiresIn) {
        this.expiresIn = expiresIn;
    }

    public long getUserId() {
        return userId;
    }

    public void setUserId(long userId) {
        this.userId = userId;
    }

    public String getDid() {
        return did;
    }

    public void setDid(String did) {
        this.did = did;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public String getEnterpriseCName() {
        return enterpriseCName;
    }

    public void setEnterpriseCName(String enterpriseCName) {
        this.enterpriseCName = enterpriseCName;
    }

    public Integer getMemberType() {
        return memberType;
    }

    public void setMemberType(Integer memberType) {
        this.memberType = memberType;
    }

    public Integer getIsOperatingCenter() {
        return isOperatingCenter;
    }

    public void setIsOperatingCenter(Integer isOperatingCenter) {
        this.isOperatingCenter = isOperatingCenter;
    }

    public AccessToken() {}

    public AccessToken(String accessToken, Long expiresIn, long userId, String did, String userName, String enterpriseCName, Integer memberType, Integer isOperatingCenter) {
        this.accessToken = accessToken;
        this.expiresIn = expiresIn;
        this.userId = userId;
        this.did = did;
        this.userName = userName;
        this.enterpriseCName = enterpriseCName;
        this.memberType = memberType;
        this.isOperatingCenter = isOperatingCenter;
    }

    public static class TokenDeserializer implements JsonDeserializer<AccessToken> {


        @Override
        public AccessToken deserialize(JsonElement jsonElement, Type type, JsonDeserializationContext context) throws JsonParseException {
            JsonObject jsonObject = jsonElement.getAsJsonObject();
            Gson gson = new Gson();
            String remainingJson = gson.toJson(jsonObject);
            // 使用 Gson 默认的 fromJson 方法处理额外字段
            AccessToken token = gson.fromJson(remainingJson, AccessToken.class);

            // 处理accessToken
            if (jsonObject.has("accessToken")) {
                token.setAccessToken(jsonObject.get("accessToken").getAsString());
            } else if (jsonObject.has("access_token")) {
                token.setAccessToken(jsonObject.get("access_token").getAsString());
            }
            // 处理expiresIn
            if (jsonObject.has("expiresIn")) {
                token.setExpiresIn(jsonObject.get("expiresIn").getAsLong());
            } else if (jsonObject.has("expires_in")) {
                token.setExpiresIn(jsonObject.get("expires_in").getAsLong());
            }

            return token;
        }

    }

    @Override
    public String toString() {
        return "AccessToken{" +
                "accessToken='" + accessToken + '\'' +
                ", expiresIn=" + expiresIn +
                ", userId=" + userId +
                ", did='" + did + '\'' +
                ", userName='" + userName + '\'' +
                ", enterpriseCName='" + enterpriseCName + '\'' +
                ", memberType=" + memberType +
                ", isOperatingCenter=" + isOperatingCenter +
                '}';
    }
}
