/*
Copyright (C) BABEC. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/
package org.chainweaver.did.sdk.model;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.gson.annotations.SerializedName;

import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * did文档
 */
public class Document {

    /**
     * 标准参考
     */
    @SerializedName("@context")
    @JsonProperty("@context")
    private List<String> context = Collections.singletonList(" https://cnbn.org.cn/ns/did/v1");

    /**
     * 有权限签名、验签的公钥key列表
     */
    private List<String> authentication;

    /**
     * 控制方
     */
    private List<String> controller;

    /**
     * 创建时间
     */
    private String created;

    /**
     * did唯一id
     */
    private String id;

    /**
     * 证明结构
     */
    private List<Proof> proof;

    /**
     * 签发者提供的服务
     */
    private List<Service> service;

    /**
     * 外部系统DID
     */
    private Map<String, String> unionId;

    /**
     * 更新时间
     */
    private String  updated;

    /**
     * 有权签名的公钥地址
     */
    private List<VerificationMethod> verificationMethod;

    public List<String> getContext() {
        return context;
    }

    public void setContext(List<String> context) {
        this.context = context;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getCreated() {
        return created;
    }

    public void setCreated(String created) {
        this.created = created;
    }

    public String getUpdated() {
        return updated;
    }

    public void setUpdated(String updated) {
        this.updated = updated;
    }

    public List<VerificationMethod> getVerificationMethod() {
        return verificationMethod;
    }

    public void setVerificationMethod(List<VerificationMethod> verificationMethod) {
        this.verificationMethod = verificationMethod;
    }

    public List<String> getAuthentication() {
        return authentication;
    }

    public void setAuthentication(List<String> authentication) {
        this.authentication = authentication;
    }

    public List<String> getController() {
        return controller;
    }

    public void setController(List<String> controller) {
        this.controller = controller;
    }

    public List<Service> getService() {
        return service;
    }

    public void setService(List<Service> service) {
        this.service = service;
    }

    public Map<String, String> getUnionId() {
        return unionId;
    }

    public void setUnionId(Map<String, String> unionId) {
        this.unionId = unionId;
    }

    public List<Proof> getProof() {
        return proof;
    }

    public void setProof(List<Proof> proof) {
        this.proof = proof;
    }

    public Document() {}

    @Override
    public String toString() {
        return "Document{" +
                "context=" + context +
                ", authentication=" + authentication +
                ", controller=" + controller +
                ", created='" + created + '\'' +
                ", id='" + id + '\'' +
                ", proof=" + proof +
                ", service=" + service +
                ", unionId=" + unionId +
                ", updated='" + updated + '\'' +
                ", verificationMethod=" + verificationMethod +
                '}';
    }
}
