/*
Copyright (C) BABEC. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/
package org.chainweaver.did.sdk.model;

import java.util.Base64;

public class Plaintext {

    /**
     * 签名原文
     */
    private String plaintext;

    /**
     * 签名
     */
    private String signature;

    /**
     * 公钥地址
     */
    private String verificationMethod;

    /**
     * 算法类型
     * Default: SM2VerificationKey2023
     */
    private String type;

    /**
     * 创建时间
     */
    private String created;

    public String getPlaintext() {
        return plaintext;
    }

    public void setPlaintext(String plaintext) {
        this.plaintext = plaintext;
    }

    public String getSignature() {
        return signature;
    }

    public void setSignature(String signature) {
        this.signature = signature;
    }

    public String getVerificationMethod() {
        return verificationMethod;
    }

    public void setVerificationMethod(String verificationMethod) {
        this.verificationMethod = verificationMethod;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getCreated() {
        return created;
    }

    public void setCreated(String created) {
        this.created = created;
    }

    public Plaintext() {
    }


    /**
     * 公钥验签入参
     * @param plaintext 签名原文
     * @param signature 签名
     * @param verificationMethod 公钥地址
     */
    public Plaintext(String plaintext, String signature, String verificationMethod) {
        this.plaintext = plaintext;
        this.signature = signature;
        this.verificationMethod = verificationMethod;
        this.type = "SM2VerificationKey2023";
    }

    /**
     * 公钥验签入参
     * @param plaintext 签名原文
     * @param signature 签名
     * @param verificationMethod 公钥地址
     * @param type 类型，默认使用SM2VerificationKey2023
     */
    public Plaintext(String plaintext, String signature, String verificationMethod, String type) {
        this.plaintext = plaintext;
        this.signature = signature;
        this.verificationMethod = verificationMethod;
        this.type = type;
    }

    /**
     * @param plaintext 签名原文
     * @param verificationMethod 公钥地址
     */
    public Plaintext(String plaintext, String verificationMethod) {
        this.plaintext = plaintext;
        this.verificationMethod = verificationMethod;
    }

    /**
     * 公钥加密入参
     * @param plaintext 签名原文
     * @param verificationMethod 公钥地址
     */
    public static Plaintext genMsgEncrypt(String plaintext, String verificationMethod) {
        return new Plaintext(Base64.getEncoder().encodeToString((plaintext.getBytes())), verificationMethod);
    }

    /**
     * 公钥加密入参
     * @param plaintext Base64加密的原文
     * @param verificationMethod 公钥地址
     */
    public static Plaintext genMsgEncryptWithBase64(String plaintext, String verificationMethod) {
        return new Plaintext(plaintext, verificationMethod);
    }

    /**
     * 私钥签名
     * @param plaintext 签名原文
     * @param verificationMethod 公钥地址
     */
    public static Plaintext genMsgSign(String plaintext, String verificationMethod) {
        return new Plaintext(Base64.getEncoder().encodeToString((plaintext.getBytes())), verificationMethod);
    }

    /**
     * 私钥签名
     * @param plaintext Base64加密的原文
     * @param verificationMethod 公钥地址
     */
    public static Plaintext genMsgSignWithBase64(String plaintext, String verificationMethod) {
        return new Plaintext(plaintext, verificationMethod);
    }

    @Override
    public String toString() {
        return "Plaintext{" +
                "plaintext='" + plaintext + '\'' +
                ", signature='" + signature + '\'' +
                ", verificationMethod='" + verificationMethod + '\'' +
                ", type='" + type + '\'' +
                ", created='" + created + '\'' +
                '}';
    }
}
