/*
Copyright (C) BABEC. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/
package org.chainweaver.did.sdk.model;

import java.util.Arrays;
import java.util.List;

/**
 * 证明结构
 */
public class Proof {


    /**
     * 签名时间
     */
    private String created;

    /**
     * 排除字段
     */
    private List<String> excludedFields;

    /**
     * 签名目的
     */
    private String proofPurpose;

    /**
     * 签名值
     */
    private String proofValue;


    /**
     * 签名某些字段
     */
    private List<String> signedFields;

    /**
     * 算法类型
     */
    private String type;

    /**
     * 公钥id
     */
    private String verificationMethod;


    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getCreated() {
        return created;
    }

    public void setCreated(String created) {
        this.created = created;
    }

    public String getProofPurpose() {
        return proofPurpose;
    }

    public void setProofPurpose(String proofPurpose) {
        this.proofPurpose = proofPurpose;
    }

    public String getVerificationMethod() {
        return verificationMethod;
    }

    public void setVerificationMethod(String verificationMethod) {
        this.verificationMethod = verificationMethod;
    }

    public String getProofValue() {
        return proofValue;
    }

    public void setProofValue(String proofValue) {
        this.proofValue = proofValue;
    }

    public List<String> getExcludedFields() {
        return excludedFields;
    }

    public void setExcludedFields(List<String> excludedFields) {
        this.excludedFields = excludedFields;
    }

    public List<String> getSignedFields() {
        return signedFields;
    }

    public void setSignedFields(List<String> signedFields) {
        this.signedFields = signedFields;
    }


    public Proof() {
    }

    /**
     * 证明结构
     * @param verificationMethod 签名公钥
     */
    public Proof(String verificationMethod) {
        this.verificationMethod = verificationMethod;
    }
    /**
     * 证明结构
     * @param type 签名算法
     * @param created 签名时间
     * @param proofPurpose 签名目的
     * @param verificationMethod 签名公钥
     */
    public Proof(String type, String created, String proofPurpose, String verificationMethod) {
        this.type = type;
        this.created = created;
        this.proofPurpose = proofPurpose;
        this.verificationMethod = verificationMethod;
    }

    /**
     * 证明结构
     * @param type 签名算法
     * @param created 签名时间
     * @param proofPurpose 签名目的
     * @param verificationMethod 签名公钥
     * @param proofValue 签名值
     */
    public Proof(String type, String created, String proofPurpose, String verificationMethod, String proofValue) {
        this.type = type;
        this.created = created;
        this.proofPurpose = proofPurpose;
        this.verificationMethod = verificationMethod;
        this.proofValue = proofValue;
    }

    @Override
    public String toString() {
        return "Proof{" +
                "created='" + created + '\'' +
                ", excludedFields=" + excludedFields +
                ", proofPurpose='" + proofPurpose + '\'' +
                ", proofValue='" + proofValue + '\'' +
                ", signedFields=" + signedFields +
                ", type='" + type + '\'' +
                ", verificationMethod='" + verificationMethod + '\'' +
                '}';
    }
}
