/*
Copyright (C) BABEC. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/
package org.chainweaver.did.sdk.model;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.gson.annotations.SerializedName;

import java.util.Collections;
import java.util.List;

/**
 * vc凭证
 */
public class VerifiableCredential {


    /**
     * 上下文
     */
    @SerializedName("@context")
    @JsonProperty("@context")
    private List<String> context = Collections.singletonList("https://cnbn.org.cn/ns/credentials/v2");;

    /**
     * 凭证详情
     */
    private Object credentialSubject;

    /**
     * 过期时间
     */
    private String expirationDate;


    /**
     * 持有人
     */
    private String holder;

    /**
     * 签发机构颁发的唯一标识
     */
    private String id;

    /**
     * 签发时间
     */
    private String issuanceDate;

    /**
     * 签发机构
     */
    private String issuer;

    /**
     * 证明结构
     */
    private List<Proof> proof;

    /**
     * 模板
     */
    private VcTemplate template;

    /**
     * 凭证类型
     */
    private List<String> type;


    public List<String> getContext() {
        return context;
    }

    public void setContext(List<String> context) {
        this.context = context;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public List<String> getType() {
        return type;
    }

    public void setType(List<String> type) {
        this.type = type;
    }

    public String getIssuer() {
        return issuer;
    }

    public void setIssuer(String issuer) {
        this.issuer = issuer;
    }

    public String getIssuanceDate() {
        return issuanceDate;
    }

    public void setIssuanceDate(String issuanceDate) {
        this.issuanceDate = issuanceDate;
    }

    public String getHolder() {
        return holder;
    }

    public void setHolder(String holder) {
        this.holder = holder;
    }

    public String getExpirationDate() {
        return expirationDate;
    }

    public void setExpirationDate(String expirationDate) {
        this.expirationDate = expirationDate;
    }

    public Object getCredentialSubject() {
        return credentialSubject;
    }

    public void setCredentialSubject(Object credentialSubject) {
        this.credentialSubject = credentialSubject;
    }

    public VcTemplate getTemplate() {
        return template;
    }

    public void setTemplate(VcTemplate template) {
        this.template = template;
    }

    public List<Proof> getProof() {
        return proof;
    }

    public void setProof(List<Proof> proof) {
        this.proof = proof;
    }

    public VerifiableCredential() {}

    @Override
    public String toString() {
        return "VerifiableCredential{" +
                "context=" + context +
                ", credentialSubject=" + credentialSubject +
                ", expirationDate='" + expirationDate + '\'' +
                ", holder='" + holder + '\'' +
                ", id='" + id + '\'' +
                ", issuanceDate='" + issuanceDate + '\'' +
                ", issuer='" + issuer + '\'' +
                ", proof=" + proof +
                ", template=" + template +
                ", type=" + type +
                '}';
    }
}
