/*
Copyright (C) BABEC. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/
package org.chainweaver.did.sdk.model.req;


import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

/**
 * 颁发vc入参
 */
public class IssueVc {

    /**
     * 凭证模版编号
     */
    private String templateId;

    /**
     * 模版填充内容
     */
    private Object credentialSubject;

    /**
     * 持证人
     */
    private String holder;

    /**
     * 过期时间
     */
    private String expirationDate;

    /**
     * 验签的公钥地址
     */
    private String verificationMethod;

    /**
     * 版本号
     */
    private String version;

    public String getTemplateId() {
        return templateId;
    }

    public void setTemplateId(String templateId) {
        this.templateId = templateId;
    }

    public Object getCredentialSubject() {
        return credentialSubject;
    }

    public void setCredentialSubject(Object credentialSubject) {
        this.credentialSubject = credentialSubject;
    }

    public String getHolder() {
        return holder;
    }

    public void setHolder(String holder) {
        this.holder = holder;
    }

    public String getExpirationDate() {
        return expirationDate;
    }

    public void setExpirationDate(String expirationDate) {
        this.expirationDate = expirationDate;
    }

    public String getVerificationMethod() {
        return verificationMethod;
    }

    public void setVerificationMethod(String verificationMethod) {
        this.verificationMethod = verificationMethod;
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    @Override
    public String toString() {
        return "IssueVc{" +
                "templateId='" + templateId + '\'' +
                ", credentialSubject=" + credentialSubject +
                ", holder='" + holder + '\'' +
                ", expirationDate='" + expirationDate + '\'' +
                ", verificationMethod='" + verificationMethod + '\'' +
                ", version='" + version + '\'' +
                '}';
    }

    public IssueVc() {
    }

    /**
     * 验证vc
     * @param templateId 凭证模版编号
     * @param credentialSubject 模版填充内容
     * @param holder 持证人
     * @param verificationMethod 验签的公钥地址
     */
    public IssueVc(String templateId, Object credentialSubject, String holder, String verificationMethod) {
        this.templateId = templateId;
        this.credentialSubject = credentialSubject;
        this.holder = holder;
        this.expirationDate = this.expirationDate();
        this.verificationMethod = verificationMethod;
    }

    private String expirationDate() {
        // 获取当前时间
        LocalDateTime currentTime = LocalDateTime.now();

        // 加上 5 分钟
        LocalDateTime modifiedTime = currentTime.plusMinutes(5);

        // 格式化成 "2024-03-06T15:06:47Z" 这种格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss'Z'");
        return modifiedTime.format(formatter);
    }
}
